
<#PSScriptInfo

.VERSION 2.0.0.0

.GUID a3ba39d8-8cbe-4c0f-87be-8ec03ffd3014

.AUTHOR Thomas Dobler - germanpowershell.com

.COMPANYNAME germanpowershell.com

.COPYRIGHT  2021 by germanpowershell.com - Alle Rechte vorbehalten

.TAGS PRTG CustomSensor PowerSHELL EXEXML

.LICENSEURI https://www.germanpowershell.com/

.PROJECTURI https://www.germanpowershell.com/

.ICONURI 

.EXTERNALMODULEDEPENDENCIES 

.REQUIREDSCRIPTS None

.EXTERNALSCRIPTDEPENDENCIES None

.RELEASENOTES
nderungsverlauf des Scripts nach dem Schema Major.Minor.Build.Revision,jeweils Major Versionen sind produktiv zu verwenden
Version     |Type      |Datum         |Benutzer            |Bemerkungen
1.2.0.0     |BUILD     |2018.09.13    |Tom Dobler          |Script bernommen aus vorgnger Version


2.0.0.0     |MAJOR     |2020.09.13    |Tom Dobler          |Float hinzugefgt bzw. verndert, Aliase werden nicht mehr untersttzt


#> 



<# 

.DESCRIPTION 
 Dieses Script bietet zwei PowerSHELL Cmdlet's zum erstellen von Custom Sensoren in PRTG mit ausgabe eines UTF-8 konformen XML Codes.

#> 
Param()


function Write-PRTGChannel{

<#
.SYNOPSIS
    Dieser PowerSHELL Befehl (Write-PRTGChannel) generiert die ntige XML Struktur eines Kanals (Channel) fr die Verwendung in einem "Custom Sensor" von PRTG.
.DESCRIPTION
    Das Cmdlet Write-PRTGChannel ist direkt im Code dokumentiert, ausserdem gibt es auf dem Kanal sowie im Blog unter www.germanpowershell.com Videos und Beschreibungen dazu.
.NOTES
    File Name  : GPS_PRTGFunktionen.ps1
    Author     : Tom Dobler - German PowerSHELL
    Email      : tom@germanpowershell.com
.VERSION
    Version 1.3
.EXAMPLE
    $outputvar += Write-PRTGChannel -Titel "Meine Werte" -Value "20"

.INPUTTYPE
   Mehrere Parameter als STRING, siehe Kommentar im Script
.RETURNVALUE
   Output type  [String][XML]
.LINK
   http://www.germanpowershell.com
.ROLE
   Paessler PRTG Custom Sensors EXE/Advanced -> www.paessler.com
.FUNCTIONALITY
   Gibt eine schnelle und valide Mglichkeit Custom Sensors zu PRTG hinzuzufgen.
#>

# Powershell Function um den geeigneten XML Output fr einen Kanal zu generieren, inkl. Validierung
# Die Werte unbedingt in eine Variable schreiben z.B. $outvar += Write-PRTGChannel usw.
# Die Ausgabe erfolg dann mit der Funktion WritePRTGOutput mit Angabe der Outputvariabel.

Param
(
# Titel des Kanals
[Parameter(Mandatory=$true)]
[string] $Titel,

# Wert des Kanals
[Parameter(Mandatory=$true)]
[ValidatePattern('\A\d*\.?\d*\Z')]
[string] $Value,

# Warnung, sobald diese Limite berschritten wird
[Parameter(Mandatory=$false)]
[ValidatePattern('\A\d*\.?\d*\Z')]
[string] $LimitMaxWarning ="",

# Error, sobald diese Limite berschritten wird
[Parameter(Mandatory=$false)]
[ValidatePattern('\A\d*\.?\d*\Z')]
[string] $LimitMaxError ="",

# Warnung, sobald dieser Wert unterschritten wird
[Parameter(Mandatory=$false)]
[ValidatePattern('\A\d*\.?\d*\Z')]
[string] $LimitMinWarning ="",

# Error, sobald dieser Wert unterschritten wird
[Parameter(Mandatory=$false)]
[ValidatePattern('\A\d*\.?\d*\Z')]
[string] $LimitMinError ="",

# Error Nachricht, oben im Sensor Titel
[Parameter(Mandatory=$false)]
[string] $LimitErrorMsg ="",

# Warnung Nachricht, oben im Sensor Titel
[Parameter(Mandatory=$false)]
[string] $LimitWarningMsg ="",

# Einheit des Kanals, oder Custom Unit sofern eigener Wert
[Parameter(Mandatory=$false)]
[ValidateSet("BytesBandwidth","BytesMemory", "BytesDisk", "Temperature", "Percent", "TimeResponse", "TimeSeconds", "TimeHours", "Count", "CPU", "BytesFile", "SpeedDisk", "SpeedNet", "Custom", "Value Lookup")] 
[string] $Unit ="Count",

# Benutzerdefinierte Einheit als String
[Parameter(Mandatory=$false)]
[string] $CustomUnit ="",

# Lookup Datei zum Umwandeln von Zahlen in String oder spezieller Validierung. Den Namen der Datei, allerdings ohne Endung .OVL angeben
[Parameter(Mandatory=$false)]
[string] $ValueLookup ="",

## Version 1.1

# Size wird verwendet um Werte darzustellen. z.B. wenn du einen Wert von 50000 hast und Kilo verwendest, steht im Kanal 50 kilo #. Standard ist one
[Parameter(Mandatory=$false)]
[ValidateSet("One","Kilo","Mega","Giga","Tera","Byte","KiloByte","MegaByte","GigaByte","TeraByte","Bit","KiloBit","MegaBit","GigaBit","TeraBit")] 
[string] $VolumeSize ="",

# Size wird verwendet um Werte darzustellen. z.B. wenn du einen Wert von 50000 hast und Kilo verwendest, steht im Kanal 50 kilo #. Standard ist one
[Parameter(Mandatory=$false)]
[ValidateSet("One","Kilo","Mega","Giga","Tera","Byte","KiloByte","MegaByte","GigaByte","TeraByte","Bit","KiloBit","MegaBit","GigaBit","TeraBit")] 
[string] $SpeedSize ="",

# Wird verwendet um die Geschwindikeit in eine Zeitliche Relation zu bringen.
[Parameter(Mandatory=$false)]
[ValidateSet("Second","Minute","Hour","Day")] 
[string] $SpeedTime ="",

# Whle ob der Wert absolut oder ein Zhler ist.
[Parameter(Mandatory=$false)]
[ValidateSet("Absolute","Difference")] 
[string] $Mode ="",

# Mit Version 1.3 abgelst
# Definiert ob die Zahl eine Ganzzahl (0) oder eine Gleitkommazahl (1) ist.
#[Parameter(Mandatory=$false)]
#[ValidateSet("0","1")] 
#[string] $Float ="",

# Dezimalmodus sofern Float auf 1 gesetzt ist. 
[Parameter(Mandatory=$false)]
[ValidateSet("Auto","All")] 
[string] $DecimalMode ="",

# Legt fest, ob der Kanal im Diagramm dargestellt werden soll
[Parameter(Mandatory=$false)]
[ValidateSet("0","1")] 
[string] $ShowChart ="",

# Legt fest, ob der Kanal in der Tabelle dargestellt werden soll
[Parameter(Mandatory=$false)]
[ValidateSet("0","1")] 
[string] $ShowTable ="",

## Version 1.3
[Switch]$Float

)

# Start XML Tag des Kanals mit Result also Resultat
$tempvar = "<result>"

# Kanal mit Namen definieren
$tempvar += "<channel>" + $Titel + "</channel>"

# Wert des Kanals fr PRTG
$tempvar += "<value>" + $Value + "</value>"

# Falls ein Limit fr Warnungen festgelgt wurde, wird das hier eingefgt
If ($LimitMaxWarning -ne "") {$tempvar += "<LimitMaxWarning>" + $LimitMaxWarning + "</LimitMaxWarning>"}

# Falls ein Limit fr Fehler festgelgt wurde, fgen wir das hier hinzu
If ($LimitMaxError -ne "") {$tempvar += "<LimitMaxError>" + $LimitMaxError + "</LimitMaxError>"}

# Das gleiche fr einen unteren Warnwert
If ($LimitMinWarning -ne "") {$tempvar += "<LimitMinWarning>" + $LimitMinWarning + "</LimitMinWarning>"}

# Und fr den unteren Fehler Wert
If ($LimitMinError -ne "") {$tempvar += "<LimitMinError>" + $LimitMinError + "</LimitMinError>"}

# Bei Fehlern kann ein individueller Text festgelegt werden
If ($LimitErrorMsg -ne "") {$tempvar += "<LimitErrorMsg>" + $LimitErrorMsg + "</LimitErrorMsg>"}

# Bei Warnungen kann auch ein invidivueller Text festgelegt werden
If ($LimitWarningMsg -ne "") {$tempvar += "<LimitWarningMsg>" + $LimitWarningMsg + "</LimitWarningMsg>"}

# Angabe der Einheit (Unit) sofern dies nicht Custom ist
If ($Unit -ne "" -and $Unit -ne "Custom") {$tempvar += "<Unit>" + $Unit + "</Unit>"}

# Hinzufgen von Unit und Customunit falls die Einheit benutzerdefiniert ist
If ($CustomUnit -ne "" -or $Unit -eq "Custom") {$tempvar += "<CustomUnit>" + $CustomUnit + "</CustomUnit>"}

# Hinzufgen von Unit und Customunit wenn die Einheit benutzerdefiniert ist
if ($CustomUnit -ne "" -and $Unit -eq "") {$tempvar += "<unit>custom</unit><CustomUnit>" + $CustomUnit + "</CustomUnit>"}

# Angabe der Lookup Datei sofern eine verwendet werden soll. Es muss der Dateiname ohne Endung (.OVL) angegeben werden.
If ($ValueLookup -ne "") {$tempvar += "<ValueLookup>" + $ValueLookup + "</ValueLookup>"}

# Wenn Limiten fr Warnungen und Fehler eingegeben wurden, muss zustzilich LimitMode auf 1 gesetzt werden.
If ($LimitMaxWarning -ne "" -or $LimitMaxError -ne "" -or $LimitMinWarning -ne "" -or $LimitMinError -ne "") {$tempvar += "<LimitMode>1</LimitMode>"}

## Version 1.1

# Wenn VolumeSize gesetzt aber SpeedSize nicht gesetzt ist
If ($VolumeSize -ne "" -and $SpeedSize -eq "") {$tempvar += "<VolumeSize>" + $VolumeSize + "</VolumeSize>"}

# Wenn SpeedSize gesetzt aber VolumeSize nicht gesetzt ist
If ($SpeedSize -ne "" -and $VolumeSize -eq "") {$tempvar += "<SpeedSize>" + $SpeedSize + "</SpeedSize>"}

# Wenn SpeedTime und SpeedSize gesetzt sind
If ($SpeedTime -ne "" -and $SpeedSize -ne "") {$tempvar += "<SpeedTime>" + $SpeedTime + "</SpeedTime>"}

# Falls Mode gesetzt ist
If ($Mode -ne "") {$tempvar += "<Mode>" + $Mode + "</Mode>"}

# Mit Version 1.3 abgelst
# Falls Float gesetzt ist
# If ($Float -ne "") {$tempvar += "<Float>" + $Float + "</Float>"}

# Falls DecimalMode gesetzt ist
If ($DecimalMode -ne "") {$tempvar += "<DecimalMode>" + $DecimalMode + "</DecimalMode>"}

# Falls ShowChart gesetzt ist
If ($ShowChart -ne "") {$tempvar += "<ShowChart>" + $ShowChart + "</ShowChart>"}

# Falls ShowTable gesetzt ist
If ($ShowTable -ne "") {$tempvar += "<ShowTable>" + $ShowTable + "</ShowTable>"}

# Abschliessendes XML Tag fr das Resultat bzw. den Kanal
if ($Float -or $DecimalMode)
    {

        $tempvar += "<float>1</float>"

    }

$tempvar += "</result>"

# Zurckgeben der XML Struktur zur Verarbeitung mit Write-PRTGOutput
return $tempvar}


# Diese Funktion schreibt den Output fr PRTG, angegeben werden muss die OutputVariabel mit dem gewnschten Text, ebenfalls ist Zusatztext im Text-Bereich mglich
function Write-PRTGOutput {

<#
.SYNOPSIS
    Dieser PowerSHELL Befehl (Write-PRTGOutput) gibt die ntige XML Struktur eines Sensors fr die Verwendung in einem "Custom Sensor" von PRTG aus.
.DESCRIPTION
    Das Cmdlet Write-PRTGOutput ist direkt im Code dokumentiert, ausserdem gibt es auf dem Kanal sowie im Blog unter www.germanpowershell.com Videos und Beschreibungen dazu.
.NOTES
    File Name  : GPS_PRTGFunktionen.ps1
    Author     : Tom Dobler - German PowerSHELL
    Email      : tom@germanpowershell.com
.VERSION
    Version 1.0
.EXAMPLE
    Write-PRTGOutput -Outputvariable $outvar -Zusatztext "Dies ist eine Information"
.INPUTTYPE
   Mehrere Parameter als STRING, siehe Kommentar im Script
.RETURNVALUE
   Output type  [String][XML] PRTG Kompatibel
.LINK
   http://www.germanpowershell.com
.ROLE
   Paessler PRTG Custom Sensors EXE/Advanced -> www.paessler.com
.FUNCTIONALITY
   Gibt eine schnelle und valide Mglichkeit Custom Sensors zu PRTG hinzuzufgen.
#>

param(
[Parameter(Mandatory=$true)]
[string] $OutputVariable,
[Parameter(Mandatory=$false)]
[string] $ZusatzText = ""
)

# Fehlermeldungne grundstzlich unterdrcken
$ErrorActionPreference = "SilentlyContinue"

# Einleitende XML Definition bzw. Version und des Encodings auf UTF8
$tempoutput = "<?xml version=`"1.0`" encoding=`"UTF-8`" ?>"

# Start XML Tag fr PRTG
$tempoutput += "<prtg>"

# Einstellen der Konsole fest auf UTF8 Kodierung, also mit Umlauten usw.
[Console]::OutputEncoding = [System.Text.Encoding]::UTF8

# Einfgen der Kanal XML Tags aus der OutputVariable
$tempoutput += $OutputVariable

# Setzen eines Textes mit optionalem Zusatztext, Standard wird angegeben wann der Sensor das letzte Mal geprft wurde mit dem Stand. Das erscheint oben unter der Sensor berschrift.
$tempoutput += (" <text>Stand: " + (Get-Date -Format "d. MMMM yyyy HH:mm:ss ") + $ZusatzText + "</text>")

# Abschliessendes XML Tag von PRTG
$tempoutput += "</prtg>"

# Ausgabe der XML Struktur in der Konsole
[Console]::WriteLine($tempoutput)

}

# Wird ab Version 2.0.0.0 nicht mehr untersttzt

<#

# Definieren eines Aliases fr WritePRTGChannel, sofern nicht vorhanden
if (!(Test-Path alias:WritePRTGChannel)){
New-Alias WritePRTGChannel -Value Write-PRTGChannel}

# Definieren eines Aliases fr WritePRTGOutput, sofern nicht vorhanden
if (!(Test-Path alias:WritePRTGOutput)){
New-Alias WritePRTGOutput -Value Write-PRTGOutput}

#>